"""
Abstract classes to use for avoiding circular imports

Created by adam on 3/17/20
"""
__author__ = 'adam'

if __name__ == '__main__':
    pass



class DiscussionType:
    pass


class BlockableActivity:
    """Defines the activity as one whose completion
    can be blocked by another student's inaction
    """
    pass