"""
Tests that the correct content will get
sent to the reviewer after the metareview is complete
Created by adam on 2/22/20
"""
__author__ = 'adam'
# todo This should be an end-to-end integration test which actually hits the server

# if __name__ == '__main__':
#     pass
#
# from CanvasHacks.SkaaSteps.SendMetareviewToReviewer import SendMetareviewToReviewer
#
#
#
# # Make sure correct students involved (have more than the 2 in db)
# # Make sure correct text is sent
# # Make sure addressing params are correct
#
#
#
# class TestOnTimeSubmissions(  TestingBase ):
#     """Checks that works properly on first run after
#     deadline on work that has been submitted
#     """
#
#     def setUp(self):
#         self.config_for_test()
#         self.dao = SqliteDAO()
#
#     def test_sends_correctly( self ):
#         """Check that each student receives the expected message
#         containing the correct student's submission
#         """
#         # Load initial content unit data
#
#         # create review assignments
#
#         # send
#
#
#         self.fail()
#
#
#     def test_assignments_stored_correctly( self ):
#         """Check that the students who have submitted the unit
#         (and none of the non-submitteers) have been paired up
#         in the expected manner, and that the pairings have been stored
#         """
#         self.fail()
#
#
# class TestLateSubmissions( unittest.TestCase ):
#     """Checks that works properly on subsequent runs after the
#     initial assignments have been sent out
#
#     """
#
#     def test_sends_correctly( self ):
#         """Check that each student receives the expected message
#         containing the correct student's submission
#         """
#         self.fail()
