"""
Created by adam on 2/22/20
"""
__author__ = 'adam'

# todo This should be an end-to-end integration test which actually hits the server

if __name__ == '__main__':
    pass

from CanvasHacks.SkaaSteps.SendReviewToReviewee import SendReviewToReviewee
